%%% Thomas A. Adams II
%%% thomas.a.adams@ntnu.no
%%% June 2020 Release
%%% This code is given without warranty or license, provided AS-IS
%%% Please site the corresponding academic journal article when citing and using this code:
%%%
%%% Christian DA, Adams TA II, et al. cDC1 coordinate innate and adaptive responses in the omentum required 
%%% for T cell priming and memory. Science Immunology 7, eabq7432 (2022). 
%%% https://doi.org/10.1126/sciimmunol.abq7432
%%%
%%% That paper is the documentation. This code is not actively maintained and may have incomplete parts.
%%%
%%% Immunized Mouse Simulation 
%%% Description: Simulates antibodies in a mouse, given initial states and
%%%              certain parameters
%%% Inputs: NA0: Initial number of A type cells [integer units]
%%%         NP0: Initial number of P type cells [pUnits]
%%%         pAPbinding: the probability constant of A and P binding per
%%%         steptime interval
%%%         pPdeath: the probability constant of P death per steptime
%%%         interval
%%%         stoptime: end the simulation in hours.
%%%         steptime: the simulation resultion in hours
%%%                         1   2  3  4  5  6  7  8  9 10 11 12  13  14  15 
%%% Outputs: dat columns: time NA NK NC ND NE NF NG NH NI NJ NP NAP NBP NDP
%%%              rows: instance
%%%
%%%

function dat = MouseSim11(NA0, NP0, pAPbinding, IntroductionRate, BPLeavingRate, pEHleaving, pIJleaving, stoptime, steptime)

  introductiontype = 0; % 0 = linear   % 1 = quadratic
  pPdeath = 0.000; % we are hardcoding this to get rid of it from the model, haven't used it in ages.
  JType = 1; % 0 is J --> ____   1 is J ---> 2J
  pBPtoPA = 0.00; % temporary
  Jto2J = 24; % hardcoding this, haven't changed it as a factor in ages. Change to 8 if desired but may need to alter the code elsewhere too

  % Set up the variables we care about
  dat = zeros(1, 15);
  %dat(1, 2) = NA0;
  dat(1, 2) = 0;
  %dat(1, 12) = NP0;
  dat(1,12) = 0;
  i = 1; % starting at row 1
  
  % vector of variables in various states, each holds the age of each cell
  APs = [];
  BPs = [];
  Cs = [];
  Ds = [];
  Es = [];
  Fs = [];
  Gs = [];
  Hs = [];
  Is = [];
  Js = [];
  Ks = []; %% These are the I's and J's that leave and keep dividing
  
  TotalAadded = 0;
  TotalPadded = 0;
  TotalPremoved = 0;
  
  % for each timestep of the simulation
  t = 0;
  while t <= stoptime
      NA = dat(i, 2); 
      NK = dat(i, 3); 
      NC = dat(i, 4); 
      ND = dat(i, 5); 
      NE = dat(i, 6); 
      NF = dat(i, 7); 
      NG = dat(i, 8); 
      NH = dat(i, 9); 
      NI = dat(i, 10); 
      NJ = dat(i, 11); 
      NP = dat(i, 12); 
      NAP = dat(i, 13); 
      NBP = dat(i, 14); 
      NDP = dat(i, 15);
      
      
      % A appears!
      % P appears!
      % better keep track of discrete stuff to make sure you don't go over
      
      if (t < IntroductionRate)          
          
        % compute the amount of A that should have been added by this point, if it were linear.
        % then add the difference integer wise as best as you can. In this
        % way, it is possible to have individual cells added every few time
        % steps when dealing with small numbers and long introduction
        % times.
       
        if (TotalAadded < NA0)
          
          if (introductiontype==0) % if linear introduction
            TotalAThisTimestep = round((t + steptime) * NA0 / IntroductionRate);            
          else %% quadratic introduction
            TotalAThisTimestep = round((t + steptime) * (t + steptime) * NA0 / (IntroductionRate * IntroductionRate));   
          end
          
          NAtoadd = max(TotalAThisTimestep - TotalAadded,0); % max function needed to prevent negative numbs resulting from next line in cases of very small NA0
          
          if (t==0 && NAtoadd == 0) % if you don't have at least 1 in the first timestep then why bother
            NAtoadd = 1;
          end
          if (NAtoadd + TotalAadded > NA0) % make sure we don't add more than we need (could happen b/c of rounding)
            NAtoadd = NA0 - TotalAadded;
          end
          NA = NA + NAtoadd;
          TotalAadded = TotalAadded + NAtoadd; 
        end
        
        if (TotalPadded < NP0)
          
          if (introductiontype==0) % if linear introduction  
            TotalPThisTimestep = round((t + steptime) * NP0 / IntroductionRate);
          else %% quadratic introduction
            TotalPThisTimestep = round((t + steptime) * (t + steptime) * NP0 / (IntroductionRate * IntroductionRate));   
          end
          
          
          NPtoadd = max(TotalPThisTimestep - TotalPadded,0); % max function needed to prevent negative numbs resulting from next line in cases of very small NP0
          if (t==0 && NPtoadd == 0) % if you don't have at least 1 in the first timestep then why bother
            NPtoadd = 1;
          end
          if (NPtoadd + TotalPadded > NP0) % make sure we don't add more than we need (could happen b/c of rounding)
            NPtoadd = NP0 - TotalPadded;
          end
          NP = NP + NPtoadd;
          TotalPadded = TotalPadded + NPtoadd; 
        end
      end
      
      
      if (t > IntroductionRate && NBP > 0 && TotalPremoved < NP0 && BPLeavingRate > 0)  
         % BP disappears! 
         % Disappearance likely 1 every few timesteps.
         TotalPThisTimestep = round(BPLeavingRate * (t - IntroductionRate + steptime) * NP0 / IntroductionRate);
         NPtoRemove = min(max([TotalPThisTimestep - TotalPremoved, 0]), NBP); % max function needed to prevent negative numbs resulting from next line in cases of very small NP0                 
         
         if (NPtoRemove > 0)
           TotalPremoved = TotalPremoved + NPtoRemove;
           if (NPtoRemove >= NBP) % shortcut if its the last one
             BPs = [];
             NBP = 0;
           else % randomly pick from the list
             
             NumGenerations = size(BPs, 2);
                               
             % Randomly pick from the list untill they are all gone.
             for PIndex = 1:NPtoRemove
             
               loopcondition = true;
               while loopcondition
                 
                 %   generate random number between 1 and NBP             
                 Target = randi([1 NBP], 1);
               
                 % find the appropriate point in the list
                 BPGeneration = 1;
                 BPCounter = 1;
                 while BPGeneration <= NumGenerations 
                   % check if our target exists in this block 
                   if (Target >= BPCounter && Target < BPCounter + BPs(1, BPGeneration) && BPs(1, BPGeneration) > 0)  
                     BPs(1, BPGeneration) = BPs(1, BPGeneration) - 1;
                     NBP = NBP - 1;                   
                     loopcondition = false;
                     break;
                   end    
                   % try the next generation    
                   BPCounter = BPCounter + BPs(1, BPGeneration);
                   BPGeneration = BPGeneration + 1;                  
                 end %% its possible we found nothing, and need another random number.                              
               end             
             end
             % clean up the list
             NewList = [];
             for GenIndex = 1 : NumGenerations
               if (BPs(1, GenIndex) > 0)
                 NewList = [NewList BPs(:, GenIndex)];
               end % otherwise, I skip it, so it's not in the list
             end
             BPs = NewList;
           end
           
           
           
%            % recalculate as a check, I don't think I need this
%            if (isempty(BPs))
%              NBP = 0;
%            else
%              NBP = sum(BPs(1, :));
%            end
         end
      end    
      
      %% Reverse transition
      % BP --> P + A
      
      if (pBPtoPA > 0)
          
          NumGenerations = size(BPs, 2);
          TotalNewTransitions = 0;
          newList = [];
          for k = 1:NumGenerations
            nTrials = BPs(1, k); 
            Age = BPs(2, k);
            Prob = pBPtoPA;   
            mu = nTrials * Prob;
            si = sqrt(nTrials * Prob * (1-Prob)); 
            NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
            if (NTransitions < 0)
              NTransitions = 0;
            end
            if (NTransitions >= nTrials)                  
              % all elements of this generation are gone so don't add it to the list
              % this implicitly deletes it from the record
              NTransitions = nTrials;
            else 
              NewRecord = [(nTrials-NTransitions); Age]; % don't do the + steptime here, that happens in a later step. otherwise this goes too fast
              newList = [newList NewRecord]; 
            end   
            TotalNewTransitions = TotalNewTransitions + NTransitions;
          end
          BPs = newList;
          if (isempty(newList))
            NBP = 0;
          else
            NBP = sum(newList(1, :));
          end
          if (TotalNewTransitions > 0) % now add to the child records for A and P        
            NA = NA + TotalNewTransitions;
            NP = NP + TotalNewTransitions;                  
          end
      end 
      
      %% P death / BP / AP death
      % Remove from consderation
      %%%% P dies  ... even P that just arrived            
      if (pPdeath > 0 && NP > 0)
        nTrials = NP;
        Prob = pPdeath;
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        Ndeaths = round(si * randn + mu);  % this is the number of Ps that will die
        if (Ndeaths < 0)
          Ndeaths = 0;
        elseif (Ndeaths >= nTrials)                  
          Ndeaths = nTrials;
        end
        NP = NP - Ndeaths;
        NDP = NDP + Ndeaths; 
      end
      
      %%% New behaviour.... AP and BP death. They just die together
      % BP death
      % Same probability as P death. BP dies completely. Counts as P death
      if (pPdeath > 0 && NBP > 0)
          NumGenerations = size(BPs, 2);
          TotalNewTransitions = 0;
          newList = [];

          for k = 1:NumGenerations
            nTrials = BPs(1, k); 
            Age = BPs(2, k);
            Prob = pPdeath;   
            mu = nTrials * Prob;
            si = sqrt(nTrials * Prob * (1-Prob)); 
            NTransitions = round(si * randn + mu);  % this is the number of BP pairs that die
            if (NTransitions < 0)
              NTransitions = 0;
            end
            if (NTransitions >= nTrials)                  
              % all elements of this generation are gone so don't add it to the list
              % this implicitly deletes it from the record
              NTransitions = nTrials;
            else 
              NewRecord = [(nTrials-NTransitions); Age]; % just put them back in the list, age is the same
              newList = [newList NewRecord]; 
            end   
            TotalNewTransitions = TotalNewTransitions + NTransitions;
          end
          BPs = newList;
          if (isempty(newList))
            NBP = 0;
          else
            NBP = sum(newList(1, :));
          end
          NDP = NDP + TotalNewTransitions; % count up the amount of P that died    
      end
      
      %% 
      % AP death
      % Same probability as P death. The P dies, but not the A. Counts as P death
      if (pPdeath > 0 && NAP > 0)
          NumGenerations = size(APs, 2);
          TotalNewTransitions = 0;
          newList = [];
          for k = 1:NumGenerations
            nTrials = APs(1, k); 
            Age = APs(2, k);
            Prob = pPdeath;   
            mu = nTrials * Prob;
            si = sqrt(nTrials * Prob * (1-Prob)); 
            NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
            if (NTransitions < 0)
              NTransitions = 0;
            end
            if (NTransitions >= nTrials)                  
              % all elements of this generation are gone so don't add it to the list
              % this implicitly deletes it from the record
              NTransitions = nTrials;
            else 
              NewRecord = [(nTrials-NTransitions); Age];
              newList = [newList NewRecord]; 
            end   
            TotalNewTransitions = TotalNewTransitions + NTransitions;
          end
          APs = newList;
          if (isempty(newList))
            NAP = 0;
          else
            NAP = sum(newList(1, :));
          end
          NDP = NDP + TotalNewTransitions; % count up the amount of P that died              
          NA = NA + TotalNewTransitions; % The A come back.
      end
      
      %% K --> 2K
      % Run binomial probabilities on each generation based on age.
      NumGenerations = size(Ks, 2);
      newList = [];
      TotalNewTransitions = 0;
      for k = 1:NumGenerations
        nTrials = Ks(1, k); 
        Age = Ks(2, k);
        
        % Prob = SplitTransitionProb(Age);   
        % Prob = J2JTransitionProb(Age);   
        Prob = J2JTransitionProb(Age + (24-Jto2J));  % this shifts the probability profile lower based on teh desired parameter 
     
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        
        if (NTransitions >= nTrials)                  
          NTransitions = nTrials;                 
        else 
          NewRecord = [(nTrials-NTransitions); Age + steptime];
          newList = [newList NewRecord]; 
        end     
        TotalNewTransitions = TotalNewTransitions + NTransitions;
      end
      Ks = newList;
      if (isempty(newList))
        NK = 0;
      else
        NK = sum(newList(1, :));
      end
      
      if (TotalNewTransitions > 0) % now add to the child record 2 for each made
        NewChildRecord = [(2*TotalNewTransitions); 0];
        Ks = [Ks NewChildRecord];
        NK = sum(Ks(1, :));
      end
      
      
                        
      %% I and J --> exits to K 
      % a percentage from each generation will leave
      %%%for I
      TotalNewK = 0;
      NumGenerations = size(Is, 2);
      newList = [];
      for k = 1:NumGenerations
        nTrials = Is(1, k); 
        Age = Is(2, k);
        Prob = pIJleaving;   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials) 
          NTransitions = nTrials;  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
        else 
          NewRecord = [(nTrials-NTransitions); Age];
          newList = [newList NewRecord]; 
        end          
        TotalNewK = TotalNewK + NTransitions;
      end
      Is = newList;
      if (isempty(newList))
        NI = 0;
      else
        NI = sum(newList(1, :));
      end
      
      %%%for J
      NumGenerations = size(Js, 2);
      newList = [];
      for k = 1:NumGenerations
        nTrials = Js(1, k); 
        Age = Js(2, k);
        Prob = pIJleaving;   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)
          NTransitions = nTrials;  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
        else 
          NewRecord = [(nTrials-NTransitions); Age];
          newList = [newList NewRecord]; 
        end 
        TotalNewK = TotalNewK + NTransitions;
      end
      Js = newList;
      if (isempty(newList))
        NJ = 0;
      else
        NJ = sum(newList(1, :));
      end
      
      %% for E
      NumGenerations = size(Es, 2);
      newList = [];
      for k = 1:NumGenerations
        nTrials = Es(1, k); 
        Age = Es(2, k);
        Prob = pEHleaving;   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)
          NTransitions = nTrials;  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
        else 
          NewRecord = [(nTrials-NTransitions); Age];
          newList = [newList NewRecord]; 
        end 
        TotalNewK = TotalNewK + NTransitions;
      end
      Es = newList;
      if (isempty(newList))
        NE = 0;
      else
        NE = sum(newList(1, :));
      end
      
       %% for F
      NumGenerations = size(Fs, 2);
      newList = [];
      for k = 1:NumGenerations
        nTrials = Fs(1, k); 
        Age = Fs(2, k);
        Prob = pEHleaving;   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)
          NTransitions = nTrials;  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
        else 
          NewRecord = [(nTrials-NTransitions); Age];
          newList = [newList NewRecord]; 
        end 
        TotalNewK = TotalNewK + NTransitions;
      end
      Fs = newList;
      if (isempty(newList))
        NF = 0;
      else
        NF = sum(newList(1, :));
      end
      
       %% for G
      NumGenerations = size(Gs, 2);
      newList = [];
      for k = 1:NumGenerations
        nTrials = Gs(1, k); 
        Age = Gs(2, k);
        Prob = pEHleaving;   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)
          NTransitions = nTrials;  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
        else 
          NewRecord = [(nTrials-NTransitions); Age];
          newList = [newList NewRecord]; 
        end 
        TotalNewK = TotalNewK + NTransitions;
      end
      Gs = newList;
      if (isempty(newList))
        NG = 0;
      else
        NG = sum(newList(1, :));
      end
      
       %% for H
      NumGenerations = size(Hs, 2);
      newList = [];
      for k = 1:NumGenerations
        nTrials = Hs(1, k); 
        Age = Hs(2, k);
        Prob = pEHleaving;   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)
          NTransitions = nTrials;  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
        else 
          NewRecord = [(nTrials-NTransitions); Age];
          newList = [newList NewRecord]; 
        end 
        TotalNewK = TotalNewK + NTransitions;
      end
      Hs = newList;
      if (isempty(newList))
        NH = 0;
      else
        NH = sum(newList(1, :));
      end
      
      % Add the K from E through J transitions
      if (TotalNewK > 0) % now add to the child record 2 for each made
        NewChildRecord = [TotalNewK; 0];
        Ks = [Ks NewChildRecord];
        NK = sum(Ks(1, :));
      end 
      
      %% J --> invisible  or J --> 2J
      % Run binomial probabilities on each generation based on age.
      NumGenerations = size(Js, 2);
      newList = [];
      TotalNewTransitions = 0;
      for k = 1:NumGenerations
        nTrials = Js(1, k); 
        Age = Js(2, k);
        
        % Prob = SplitTransitionProb(Age);   
        % Prob = J2JTransitionProb(Age);   
        Prob = J2JTransitionProb(Age + (24-Jto2J));  % this shifts the probability profile lower based on teh desired parameter 

        
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        
                

       
        if (NTransitions >= nTrials)                  
          if (JType == 1) % J --> 2J
            NTransitions = nTrials;  
          else % J --> ____
            % all elements of this generation are gone so don't add it to the list
            % this implicitly deletes it from the record
          end                    
        else 
          NewRecord = [(nTrials-NTransitions); Age + steptime];
          newList = [newList NewRecord]; 
        end     
        if (JType == 1) % J --> 2J
          TotalNewTransitions = TotalNewTransitions + NTransitions;
        end
      end
      Js = newList;
      if (isempty(newList))
        NJ = 0;
      else
        NJ = sum(newList(1, :));
      end
      
      % for J-->2J only
      if (JType == 1 && TotalNewTransitions > 0) % now add to the child record 2 for each made
        NewChildRecord = [(2*TotalNewTransitions); 0];
        Js = [Js NewChildRecord];
        NJ = sum(Js(1, :));
      end
      
      %% I -> 2J transition      
      NumGenerations = size(Is, 2);
      TotalNewTransitions = 0;
      newList = [];
      for k = 1:NumGenerations
        nTrials = Is(1, k); 
        Age = Is(2, k);
        Prob = SplitTransitionProb(Age);   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)                  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
          NTransitions = nTrials;
        else 
          NewRecord = [(nTrials-NTransitions); Age + steptime];
          newList = [newList NewRecord]; 
        end   
        TotalNewTransitions = TotalNewTransitions + NTransitions;
      end
      Is = newList;
      if (isempty(newList))
        NI = 0;
      else
        NI = sum(newList(1, :));
      end
      if (TotalNewTransitions > 0) % now add to the child record 2 for each made
        NewChildRecord = [(2*TotalNewTransitions); 0];
        Js = [Js NewChildRecord];
        NJ = sum(Js(1, :));
      end

      %% H -> 2I transition      
      NumGenerations = size(Hs, 2);
      TotalNewTransitions = 0;
      newList = [];
      for k = 1:NumGenerations
        nTrials = Hs(1, k); 
        Age = Hs(2, k);
        Prob = SplitTransitionProb(Age);   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)                  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
          NTransitions = nTrials;
        else 
          NewRecord = [(nTrials-NTransitions); Age + steptime];
          newList = [newList NewRecord]; 
        end   
        TotalNewTransitions = TotalNewTransitions + NTransitions;
      end
      Hs = newList;
      if (isempty(newList))
        NH = 0;
      else
        NH = sum(newList(1, :));
      end
      if (TotalNewTransitions > 0) % now add to the child record 2 for each made
        NewChildRecord = [(2*TotalNewTransitions); 0];
        Is = [Is NewChildRecord];
        NI = sum(Is(1, :));
      end
      
      %% G -> 2H transition      
      NumGenerations = size(Gs, 2);
      TotalNewTransitions = 0;
      newList = [];
      for k = 1:NumGenerations
        nTrials = Gs(1, k); 
        Age = Gs(2, k);
        Prob = SplitTransitionProb(Age);   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)                  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
          NTransitions = nTrials;
        else 
          NewRecord = [(nTrials-NTransitions); Age + steptime];
          newList = [newList NewRecord]; 
        end   
        TotalNewTransitions = TotalNewTransitions + NTransitions;
      end
      Gs = newList;
      if (isempty(newList))
        NG = 0;
      else
        NG = sum(newList(1, :));
      end
      if (TotalNewTransitions > 0) % now add to the child record 2 for each made
        NewChildRecord = [(2*TotalNewTransitions); 0];
        Hs = [Hs NewChildRecord];
        NH = sum(Hs(1, :));
      end
      
            
      %% F -> 2G transition      
      NumGenerations = size(Fs, 2);
      TotalNewTransitions = 0;
      newList = [];
      for k = 1:NumGenerations
        nTrials = Fs(1, k); 
        Age = Fs(2, k);
        Prob = SplitTransitionProb(Age);   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)                  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
          NTransitions = nTrials;
        else 
          NewRecord = [(nTrials-NTransitions); Age + steptime];
          newList = [newList NewRecord]; 
        end   
        TotalNewTransitions = TotalNewTransitions + NTransitions;
      end
      Fs = newList;
      if (isempty(newList))
        NF = 0;
      else
        NF = sum(newList(1, :));
      end
      if (TotalNewTransitions > 0) % now add to the child record 2 for each made
        NewChildRecord = [(2*TotalNewTransitions); 0];
        Gs = [Gs NewChildRecord];
        NG = sum(Gs(1, :));
      end
      
      %% E -> 2F transition      
      NumGenerations = size(Es, 2);
      TotalNewTransitions = 0;
      newList = [];
      for k = 1:NumGenerations
        nTrials = Es(1, k); 
        Age = Es(2, k);
        Prob = SplitTransitionProb(Age);   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)                  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
          NTransitions = nTrials;
        else 
          NewRecord = [(nTrials-NTransitions); Age + steptime];
          newList = [newList NewRecord]; 
        end   
        TotalNewTransitions = TotalNewTransitions + NTransitions;
      end
      Es = newList;
      if (isempty(newList))
        NE = 0;
      else
        NE = sum(newList(1, :));
      end
      if (TotalNewTransitions > 0) % now add to the child record 2 for each made
        NewChildRecord = [(2*TotalNewTransitions); 0];
        Fs = [Fs NewChildRecord];
        NF = sum(Fs(1, :));
      end
      
      
      %% D -> 2E transition      
      NumGenerations = size(Ds, 2);
      TotalNewTransitions = 0;
      newList = [];
      for k = 1:NumGenerations
        nTrials = Ds(1, k); 
        Age = Ds(2, k);
        Prob = SplitTransitionProb(Age);   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)                  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
          NTransitions = nTrials;
        else 
          NewRecord = [(nTrials-NTransitions); Age + steptime];
          newList = [newList NewRecord]; 
        end   
        TotalNewTransitions = TotalNewTransitions + NTransitions;
      end
      Ds = newList;
      if (isempty(newList))
        ND = 0;
      else
        ND = sum(newList(1, :));
      end
      if (TotalNewTransitions > 0) % now add to the child record 2 for each made
        NewChildRecord = [(2*TotalNewTransitions); 0];
        Es = [Es NewChildRecord];
        NE = sum(Es(1, :));
      end
      
      
      %% C -> 2D transition      
      NumGenerations = size(Cs, 2);
      TotalNewTransitions = 0;
      newList = [];
      for k = 1:NumGenerations
        nTrials = Cs(1, k); 
        Age = Cs(2, k);
        Prob = SplitTransitionProb(Age);   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)                  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
          NTransitions = nTrials;
        else 
          NewRecord = [(nTrials-NTransitions); Age + steptime];
          newList = [newList NewRecord]; 
        end   
        TotalNewTransitions = TotalNewTransitions + NTransitions;
      end
      Cs = newList;
      if (isempty(newList))
        NC = 0;
      else
        NC = sum(newList(1, :));
      end
      if (TotalNewTransitions > 0) % now add to the child record 2 for each made
        NewChildRecord = [(2*TotalNewTransitions); 0];
        Ds = [Ds NewChildRecord];
        ND = sum(Ds(1, :));
      end
      
      
      
      
      %% check all of the BPs for BP-->P + 2C transition
      NumGenerations = size(BPs, 2);
      TotalNewTransitions = 0;
      newList = [];
      for k = 1:NumGenerations
        nTrials = BPs(1, k); 
        Age = BPs(2, k);
        Prob = BPTransitionProb(Age);   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)                  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
          NTransitions = nTrials;
        else 
          NewRecord = [(nTrials-NTransitions); Age + steptime];
          newList = [newList NewRecord]; 
        end   
        TotalNewTransitions = TotalNewTransitions + NTransitions;
      end
      BPs = newList;
      if (isempty(newList))
        NBP = 0;
      else
        NBP = sum(newList(1, :));
      end
      if (TotalNewTransitions > 0) % now add to the child records for those made
        NewChildRecord = [(2*TotalNewTransitions); 0];
        Cs = [Cs NewChildRecord];
        NC = sum(Cs(1, :));
        % uses old style for P
        NP = NP + TotalNewTransitions;
      end
      
      
      
      %% check all of the APs for AP-->BP transition
      NumGenerations = size(APs, 2);
      TotalNewTransitions = 0;
      newList = [];
      for k = 1:NumGenerations
        nTrials = APs(1, k); 
        Age = APs(2, k);
        Prob = APTransitionProb(Age);   
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        NTransitions = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (NTransitions < 0)
          NTransitions = 0;
        end
        if (NTransitions >= nTrials)                  
          % all elements of this generation are gone so don't add it to the list
          % this implicitly deletes it from the record
          NTransitions = nTrials;
        else 
          NewRecord = [(nTrials-NTransitions); Age + steptime];
          newList = [newList NewRecord]; 
        end   
        TotalNewTransitions = TotalNewTransitions + NTransitions;
      end
      APs = newList;
      if (isempty(newList))
        NAP = 0;
      else
        NAP = sum(newList(1, :));
      end
      if (TotalNewTransitions > 0) % now add to the child record for each made
        NewChildRecord = [(TotalNewTransitions); 0];
        BPs = [BPs NewChildRecord];
      end
      if (isempty(BPs))
        NBP = 0;
      else
        NBP = sum(BPs(1, :));
      end
      
      %% check for A + P binding    
      % A + P --> AP transition using normal distributions
      if (NA > 0)
        % Prob = pAPbinding*(NA / (NA + NAP + NBP + NP + NC + ND + NE + NF + NG + NH + NI + NJ));   
        Prob = pAPbinding*(NA / (NA + NP)); 
        if (Prob >= 1)
            Prob = 1;
        end
          
        nTrials = NP;
        mu = nTrials * Prob;
        si = sqrt(nTrials * Prob * (1-Prob)); 
        TotalCreated = round(si * randn + mu);  % this is the number of A+Ps that bind
        if (TotalCreated < 0)
          TotalCreated = 0;
        end
        if TotalCreated > NP % its possible with luck and rounding to go over by 1
          TotalCreated = NP;
        end
        if TotalCreated > NA % should not happen but in case A > P, just check.
          TotalCreated = NA;
        end
        NA = NA - TotalCreated;
        NP = NP - TotalCreated; 
        APs = [APs [TotalCreated; 0]];
        NAP = sum(APs(1,:));
      end    

                        
      i = i + 1; % next iteration
      t = t + steptime; % next time step
      
      %% store all of the relevant data
      dat(i, 1) = t;
      dat(i, 2) = NA; 
      dat(i, 3) = NK; 
      dat(i, 4) = NC; 
      dat(i, 5) = ND; 
      dat(i, 6) = NE; 
      dat(i, 7) = NF; 
      dat(i, 8) = NG; 
      dat(i, 9) = NH; 
      dat(i, 10) = NI; 
      dat(i, 11) = NJ; 
      dat(i, 12) = NP; 
      dat(i, 13) = NAP; 
      dat(i, 14) = NBP; 
      dat(i, 15) = NDP;
      
  end

end